@@ -26,11 +26,13 @@ pub trait Paginator<T, U> {
 pub trait PaginatedRequest: Request {
     /// Associated data that can be used for pagination.
     type Data: Clone;
+
     /// The paginator used for the request.
     type Paginator: Paginator<Self::Response, <Self as PaginatedRequest>::Data>;
 
     /// Return the associated paginator.
     fn paginator(&self) -> Self::Paginator;
+
     /// Specify the initial page to start pagination from. Defaults to `None`, which means
     /// pagination will begin from whatever page the API defines as the initial page.
     fn initial_page(&self) -> Option<<Self as PaginatedRequest>::Data> {
@@ -88,16 +90,12 @@ pub mod query {
 
     /// A paginator that implements pagination through one or more query parameters.
     pub struct QueryPaginator<T, U> {
-        f: Box<dyn Fn(Option<&U>, &T) -> Option<U>>,
-        _phantom: std::marker::PhantomData<T>,
+        f: Box<dyn 'static + Send + Sync + Fn(Option<&U>, &T) -> Option<U>>,
     }
 
     impl<T, U> QueryPaginator<T, U> {
-        pub fn new<F: 'static + Fn(Option<&U>, &T) -> Option<U>>(f: F) -> Self {
-            Self {
-                f: Box::new(f),
-                _phantom: std::marker::PhantomData,
-            }
+        pub fn new<F: 'static + Send + Sync + Fn(Option<&U>, &T) -> Option<U>>(f: F) -> Self {
+            Self { f: Box::new(f) }
         }
     }
 
@@ -166,16 +164,12 @@ pub mod path {
     /// the paginator should return the path segment number and the new path segment, e.g. (2, "foo")
     /// represents changing the third path segment to "foo"
     pub struct PathPaginator<T, U> {
-        f: Box<dyn Fn(Option<&U>, &T) -> Option<U>>,
-        _phantom: std::marker::PhantomData<T>,
+        f: Box<dyn 'static + Send + Sync + Fn(Option<&U>, &T) -> Option<U>>,
     }
 
     impl<T, U> PathPaginator<T, U> {
-        pub fn new<F: 'static + Fn(Option<&U>, &T) -> Option<U>>(f: F) -> Self {
-            Self {
-                f: Box::new(f),
-                _phantom: std::marker::PhantomData,
-            }
+        pub fn new<F: 'static + Send + Sync + Fn(Option<&U>, &T) -> Option<U>>(f: F) -> Self {
+            Self { f: Box::new(f) }
         }
     }
 
@@ -196,3 +190,28 @@ pub mod path {
         }
     }
 }
+
+//enum PaginationFn<T, U> {
+//    NoResult(Box<dyn Send + Sync + Fn(Option<&U>) -> Option<U>>),
+//    ResultNeeded(Box<dyn Send + Sync + Fn(Option<&U>, &T) -> Option<U>>),
+//}
+//
+//impl<'u, T, U: 'u> From<fn(Option<&'u U>) -> Option<U>> for PaginationFn<T, U> {
+//    fn from(x: fn(Option<&'u U>) -> Option<U>) -> PaginationFn<T, U> {
+//        PaginationFn::NoResult(Box::new(x))
+//    }
+//}
+//
+//impl<T: 'static, U: 'static> From<fn(Option<&U>, &T) -> Option<U>> for PaginationFn<T, U> {
+//    fn from(x: fn(Option<&U>, &T) -> Option<U>) -> PaginationFn<T, U> {
+//        PaginationFn::ResultNeeded(Box::new(x))
+//    }
+//}
+//
+//fn test(x: Option<&i32>) -> Option<i32> {
+//    x.cloned()
+//}
+//
+//fn new_pagination<T>() -> PaginationFn<T, i32> {
+//    test.into()
+//}
